/****************************************************************************
**	File:	LanSnoop.c     (Diagnostic responder information collector)
**
**	Desc:	Queries the diagnostic responder of the user object selected.
**
**			This code demonstrates how the diagnostic responder functions can	
**			be used. The user interface is C-Worthy. The executable is 
**			supplied in the case that this code will be used as a utility only.		
**			This code also demonstrates how to utilize the GNMA functions. If
**			a node has the NMR.VLM loaded this code will detect that case and
**			provide another option to the main menu - Network Managment Info.
**		
**		
**
**		DISCLAIMER  
**  
**	Novell, Inc. makes no representations or warranties with respect to
**	any NetWare software, and specifically disclaims any express or
**	implied warranties of merchantability, title, or fitness for a
**	particular purpose.  
**
**	Distribution of any NetWare software is forbidden without the
**	express written consent of Novell, Inc.  Further, Novell reserves
**	the right to discontinue distribution of any NetWare software.
**	
**	Novell is not responsible for lost profits or revenue, loss of use
**	of the software, loss of data, costs of re-creating lost data, the
**	cost of any substitute equipment or program, or claims by any party
**	other than you.  Novell strongly recommends a backup be made before
**	any software is installed.   Technical support for this software
**	may be provided at the discretion of Novell.
**
**	Programmers:
**
**		Ini	Who						Firm
**		-----------------------------------------------------------------------
**		KLB	Karl Bunnell				Novell Developer Support.
**
**	History:
**
** 	When     Who     What
** 	------------------------------------------------------------
** 	10-12-92 KLB     Conseption
** 	11-29-93 KLB	  Began adding Help
** 	01-11-93 KLB     Ported this utility to the new API's.
*/

/****************************************************************************
**	Include headers, macros, function prototypes, etc.
*/


	/*--------------------------------------------------------------------
	**	MSC Include Files
	*/
	#include <stdio.h>
	#include <stdlib.h>

	/*--------------------------------------------------------------------
	**	C-Worthy include files
	*/
	#pragma pack(2)
	#define MSC500
	#include <cwgen.h>
	#include <cwdos.h>
	#include <cwuser.h>
	#include <usersys.h>
	#include <userform.h>
	#include <sys$err.elh>
	#include <sys$msg.mlh>
	#include "LANSNOOP.mlh"
   #include "LANSNOOP.hlh"
	#include "LANSNOOP.elh"
	#include "LANSNOOP.h"
	#pragma pack()


	/*---------------------------------------------------------------------
	**	NetWare API include files
	*/
	#define MSC    510
	#define LINT_ARGS
	#define NWDOS
	#undef ConvertToUpperCase
	#include <nwcalls.h>
	#include <diag.h>
	

/***************************************************************************
** Global Storage.
*/

BeginDiagnosticStruct   destination;
NWINET_ADDR  				internetAddr;
BYTE                    componentList[54];
WORD                    connectionID;
WORD							currentConnectionID;
NWCONN_HANDLE				cHandle;


/**************************************************************************
** Program Shut-down.
*/
void _FASTCALL Uninit(void)
	{
	/*----------------------------------------------------------------------
	** Close down C-worthy.
	*/
	RestoreUtility();

	exit(0);
	}

/**************************************************************************
** MLID Statistics
*/

void MLIDConfig(WORD MLIDid)
{
	int							 kstat;
	int                      curportal;
	int                      oldportal;
	int							 ccode;
	BYTE                     offset;
	char                     txtbuff[80];
	MLIDConfigurationStruct	 MLIDdata;
	AllResponseData          intvptr;
	char 							 key[5];
		  
	offset = (BYTE) FindComponentOffset(componentList, 9);
	ccode =  GetMLIDConfiguration(connectionID, offset, MLIDid, &intvptr, &MLIDdata);

	curportal = CreatePortal(
    	/* topline         */ 3,
    	/* leftColumn      */ 5,
    	/* frameHeight     */ 22,
    	/* frameWidth      */ 65,
    	/* virtualHeight   */ 22,
    	/* virtualWidth    */ 65,
    	/* saveFlag        */ SAVE,
    	/* headerText      */ GetMessage(MLID_STATS_DISPLAY),
    	/* headerAttr      */ VNORMAL,
    	/* borderType      */ SINGLE,
    	/* borderAttr      */ VINTENSE,
    	/* cursorState     */ CURSOR_OFF,
    	/* directFlag      */ VIRTUAL
    	);

   oldportal=DeselectPortal();
   SelectPortal(curportal);

   do
		{

   	ClearPortal();

   	DisplayTextInPortal(0,5,"Node Address          : ",VNORMAL)  ;
   	DisplayTextInPortal(1,5,"Board Number          : ",VNORMAL);
   	DisplayTextInPortal(2,5,"Board Instance        : ",VNORMAL);
   	DisplayTextInPortal(3,5,"Maximum Packet Size   : ",VNORMAL);
   	DisplayTextInPortal(4,5,"BestDataSize          : ",VNORMAL);
   	DisplayTextInPortal(5,5,"WorstDataSize         : ",VNORMAL);
   	DisplayTextInPortal(6,5,"NIC Long Name         : ",VNORMAL);
   	DisplayTextInPortal(7,5,"NIC Short Name        : ",VNORMAL);
   	DisplayTextInPortal(8,5,"Frame Type String     : ",VNORMAL);
   	DisplayTextInPortal(9,5,"Frame Type ID         : ",VNORMAL);
   	DisplayTextInPortal(10,5,"Transport Time        : ",VNORMAL);
   	DisplayTextInPortal(11,5,"Source Route Handler  : ",VNORMAL);
   	DisplayTextInPortal(12,5,"Look Ahead Size       :",VNORMAL);
   	DisplayTextInPortal(13,5,"Line Speed            : ",VNORMAL);
   	DisplayTextInPortal(14,5,"Queue Depth           :",VNORMAL);
   	DisplayTextInPortal(15,5,"Driver Major Version  :",VNORMAL);
   	DisplayTextInPortal(16,5,"Driver Minor Version  :",VNORMAL);
   	DisplayTextInPortal(17,5,"Send Retries          :",VNORMAL);
	
   	sprintf(txtbuff, "%02X%02X%02X%02X%02X%02X",
			MLIDdata.NodeAddress[0],
		   MLIDdata.NodeAddress[1],	
		   MLIDdata.NodeAddress[2],	
		   MLIDdata.NodeAddress[3],	
		   MLIDdata.NodeAddress[4],	
			MLIDdata.NodeAddress[5]);

   	DisplayTextInPortal(0,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.BoardNumber);
   	DisplayTextInPortal(1,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.BoardInstance);
   	DisplayTextInPortal(2,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.MaxPacketSize);
   	DisplayTextInPortal(3,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.BestDataSize);
   	DisplayTextInPortal(4,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u", MLIDdata.WorstDataSize);
   	DisplayTextInPortal(5,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%s",MLIDdata.NICLongName);
		txtbuff[0] = ' ';
   	DisplayTextInPortal(6,29,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%s",MLIDdata.NICShortName);
		txtbuff[0] = ' ';
   	DisplayTextInPortal(7,29,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%s",MLIDdata.FrameTypeString);
		txtbuff[0] = ' ';
   	DisplayTextInPortal(8,29,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.FrameTypeID);
   	DisplayTextInPortal(9,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.TransportTime);
   	DisplayTextInPortal(10,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%LX",MLIDdata.SourceRouteHandler);
   	DisplayTextInPortal(11,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.LookAheadSize);
   	DisplayTextInPortal(12,30,txtbuff,VNORMAL);

		sprintf(txtbuff, "%u",MLIDdata.LineSpeed);
   	DisplayTextInPortal(13,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.QueueDepth);
   	DisplayTextInPortal(14,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.DriverMajorVersion);
   	DisplayTextInPortal(15,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.DriverMinorVersion);
   	DisplayTextInPortal(16,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.SendRetries);
   	DisplayTextInPortal(17,30,txtbuff,VNORMAL);

   	UpdatePortal();
		WaitForAnyKey();
		ClearPortal();

   	DisplayTextInPortal(0,5,"Config. Table Link    : ",VNORMAL);
   	DisplayTextInPortal(1,5,"Slot                  : ",VNORMAL);
   	DisplayTextInPortal(2,5,"IO Address #1         : ",VNORMAL);
   	DisplayTextInPortal(3,5,"IO Range   #1         : ",VNORMAL);
   	DisplayTextInPortal(4,5,"IO Address #2         : ",VNORMAL);
   	DisplayTextInPortal(5,5,"IO Range   #2         : ",VNORMAL);
   	DisplayTextInPortal(6,5,"Memory Address #1     : ",VNORMAL);
   	DisplayTextInPortal(7,5,"Memory Size    #1     : ",VNORMAL);
   	DisplayTextInPortal(8,5,"Memory Address #2     : ",VNORMAL);
   	DisplayTextInPortal(9,5,"Memory Size    #2     : ",VNORMAL);
   	DisplayTextInPortal(10,5,"Interrupt Line #1     : ",VNORMAL);
   	DisplayTextInPortal(11,5,"Interrupt Line #2     : ",VNORMAL);
   	DisplayTextInPortal(12,5,"DMA Line #1           :",VNORMAL);
   	DisplayTextInPortal(13,5,"DMA Line #2           : ",VNORMAL);

   	sprintf(txtbuff, "%LX",MLIDdata.ConfigTableLink);
   	DisplayTextInPortal(0,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.Slot);
   	DisplayTextInPortal(1,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%04X",MLIDdata.IOAddress1);
   	DisplayTextInPortal(2,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%04X",MLIDdata.IORange1);
   	DisplayTextInPortal(3,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%04X",MLIDdata.IOAddress2);
   	DisplayTextInPortal(4,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%04X", MLIDdata.IORange2);
   	DisplayTextInPortal(5,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%08X",MLIDdata.MemoryAddress1);
   	DisplayTextInPortal(6,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.MemorySize1);
   	DisplayTextInPortal(7,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%08X",MLIDdata.MemoryAddress2);
   	DisplayTextInPortal(8,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.MemorySize2);
   	DisplayTextInPortal(9,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.IntLine1);
   	DisplayTextInPortal(10,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.IntLine2);
   	DisplayTextInPortal(11,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.DMALine1);
   	DisplayTextInPortal(12,30,txtbuff,VNORMAL);

		sprintf(txtbuff, "%u",MLIDdata.DMALine2);
   	DisplayTextInPortal(13,30,txtbuff,VNORMAL);


   	UpdatePortal();

		kstat = WaitForFunctionKey(key);

   	} while(*(key) != I_ESC);

	ReselectPortal(oldportal);
	DestroyPortal(curportal);
		

}

/**************************************************************************
** MLID Statistics Action Procedure 
*/
int MLIDStatsAct(int operation, LIST **element, int *line)
	{
	LIST    *node;
	MLIDID  *mlidID;

	node =  *element;

	mlidID =(MLIDID *)node->otherInfo;

	switch(operation)
	   {
	   case M_ESCAPE:
	      return(0);

		case M_SELECT:
			MLIDConfig(mlidID->MLIDIdnt);
			break;
	   }

	return(-1);
	}



/**************************************************************************
** Build MLID List
*/
void MLIDInfo(void)
	{
	int							 ccode;
	WORD							 count;
	WORD							 numMLIDIDs;
	BYTE                     offset;
	MLIDID						 *mlidID;
	MLIDListStruct				 MLIDListData;
	AllResponseData          intvptr;
	char							 nameBuff[18];

	PushList();
	InitList();

	offset = (BYTE) FindComponentOffset(componentList, 9);
	ccode =  GetMLIDList(connectionID, offset, &intvptr, &MLIDListData);

	numMLIDIDs = MLIDListData.NumberOfMLIDs;

	for(count=0; count<numMLIDIDs; count++)
		{
			mlidID = malloc(sizeof(MLIDID));
			if(mlidID == NULL) break;
			mlidID->MLIDIdnt = MLIDListData.MLID[count].MLIDId;
			sprintf(nameBuff, "%s_%d",MLIDListData.MLID[count].Name,count);
			nameBuff[0] = ' ';
		   AppendToList(nameBuff, (char *)mlidID);
		}

	ccode = List(
		/* Header Text   */ MLID_LIST,
		/* Center Line   */ 0,
		/* Center Col    */ 15,
		/* Visible High  */ 10,
		/* Visible Width */ 25,
		/* Valid ex keys */ M_ESCAPE|M_SELECT,
		/* Element **    */ NullPointer(LIST),
		/* split         */ 0,
		/* Format Proc   */ NullProcedure(int),
		/* Action Proc   */ MLIDStatsAct
		);

	DestroyList();
	PopList();

	}


/**************************************************************************
** Protocol Statistics
*/
void ProtocolStats(WORD stackID)
	{
	int                             curportal;
	int                             oldportal;
	char                            txtbuff[80];
	int                             ccode;
	AllResponseData                 intvptr;
	ProtocolStackConfigStruct		  pConfigData;
	ProtocolStackStatsStruct		  pStackData;
	BYTE                            offset;

	offset = (BYTE) FindComponentOffset(componentList, 9);
	ccode = GetProtocolStackConfiguration(connectionID, offset, stackID, &intvptr, &pConfigData);
	offset = (BYTE) FindComponentOffset(componentList, 9);
	ccode = GetProtocolStackStatistics(connectionID, offset, stackID, &intvptr, &pStackData);



	curportal = CreatePortal(
   /* topline         */ 5,
   /* leftColumn      */ 5,
   /* frameHeight     */ 16,
   /* frameWidth      */ 70,
   /* virtualHeight   */ 16,
   /* virtualWidth    */ 70,
   /* saveFlag        */ SAVE,
   /* headerText      */ GetMessage(PROTOCOL_STATS_DISPLAY),
   /* headerAttr      */ VNORMAL,
   /* borderType      */ SINGLE,
   /* borderAttr      */ VINTENSE,
   /* cursorState     */ CURSOR_OFF,
   /* directFlag      */ VIRTUAL
   );
   oldportal=DeselectPortal();
   SelectPortal(curportal);
   ClearPortal();


   DisplayTextInPortal(0,5,"Configuration Major Version: ",VNORMAL);
   DisplayTextInPortal(1,5,"Configuration Minor Version: ",VNORMAL);
   DisplayTextInPortal(2,5,"Protocol Long Name         : ",VNORMAL);
   DisplayTextInPortal(3,5,"Protocol Short Name        : ",VNORMAL);
   DisplayTextInPortal(4,5,"Stack Major Version        : ",VNORMAL);
   DisplayTextInPortal(5,5,"Stack Minor Version        : ",VNORMAL);
   DisplayTextInPortal(6,5,"Statistics Major Version   : ",VNORMAL);
   DisplayTextInPortal(7,5,"Statistics Minor Version   : ",VNORMAL);
   DisplayTextInPortal(8,5,"Generic Counters           : ",VNORMAL);
   DisplayTextInPortal(9,5,"Total Transmitted Packets  : ",VNORMAL);
   DisplayTextInPortal(10,5,"Total Received Packets     : ",VNORMAL);
   DisplayTextInPortal(11,5,"Ignored Received Packets   : ",VNORMAL);

	sprintf(txtbuff, "%d", pConfigData.ConfigMajorVersion);
   DisplayTextInPortal(0,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%d",pConfigData.ConfigMinorVersion);
   DisplayTextInPortal(1,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%s",pConfigData.ProtocolLongName);
	txtbuff[0] = ' ';
   DisplayTextInPortal(2,34,txtbuff,VNORMAL);

   sprintf(txtbuff, "%s",pConfigData.ProtocolShortName);
	txtbuff[0] = ' ';
   DisplayTextInPortal(3,34,txtbuff,VNORMAL);

   sprintf(txtbuff, "%d",pConfigData.StackMajorVersion);
   DisplayTextInPortal(4,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%d", pConfigData.StackMinorVersion);
   DisplayTextInPortal(5,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%d",pStackData.StatMajorVersion);
   DisplayTextInPortal(6,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%d",pStackData.StatMinorVersion);
   DisplayTextInPortal(7,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%u",pStackData.GenericCounters);
   DisplayTextInPortal(8,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%ld",pStackData.TotalTxPackets);
   DisplayTextInPortal(9,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%ld",pStackData.TotalRxPackets);
   DisplayTextInPortal(10,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%ld",pStackData.IgnoredRxPackets);
   DisplayTextInPortal(11,35,txtbuff,VNORMAL);


   UpdatePortal();
   WaitForEscape();
	ReselectPortal(oldportal);
	DestroyPortal(curportal);
	
	return;
	}

/**************************************************************************
** Protocal Statistics Action Procedure 
*/
int ProtocolStatsAct(int operation, LIST **element, int *line)
	{
	LIST    *node;
	PSTACKID  *pStckID;

	node =  *element;

	pStckID =(PSTACKID *)node->otherInfo;

	switch(operation)
	   {
	   case M_ESCAPE:
	      return(0);

		case M_SELECT:
			ProtocolStats(pStckID->pStackID);
			break;
	   }

	return(-1);
	}



/**************************************************************************
** Build Protocol List
*/
void ProtocolInfo(void)
	{
	int							 ccode;
	WORD							 count;
	WORD							 numOfProIDs;
	BYTE                     offset;
	PSTACKID						 *stckID;
	ProtocolStackListStruct  pListData;
	AllResponseData          intvptr;
	char							 nameBuff[18];

	PushList();
	InitList();

	offset = (BYTE) FindComponentOffset(componentList, 9);
	ccode =  GetProtocolStackList(connectionID, offset, &intvptr, &pListData);

	numOfProIDs = pListData.NumberOfStacks;

	for(count=0; count<numOfProIDs; count++)
		{
			stckID = malloc(sizeof(PSTACKID));
			if(stckID == NULL) break;
			stckID->pStackID = pListData.PSID[count].StackId;
			sprintf(nameBuff, "%s",pListData.PSID[count].Name);
			nameBuff[0] = ' ';
		   AppendToList(nameBuff, (char *)stckID);
		}

	ccode = List(
		/* Header Text   */ PROTOCOL_LIST,
		/* Center Line   */ 0,
		/* Center Col    */ 15,
		/* Visible High  */ 10,
		/* Visible Width */ 25,
		/* Valid ex keys */ M_ESCAPE|M_SELECT,
		/* Element **    */ NullPointer(LIST),
		/* split         */ 0,
		/* Format Proc   */ NullProcedure(int),
		/* Action Proc   */ ProtocolStatsAct
		);

	DestroyList();
	PopList();

	}

/**************************************************************************
** Displays Machine's Static information (GNMA Function).
*/

void WSStaticStat(void)
{
	int							 kstat;
	int                      curportal;
	int                      oldportal;
	int							 ccode;
	BYTE                     offset;
	char                    txtbuff[80];
	MachineStaticInfoStruct	 Statdata;
	AllResponseData          intvptr;
	char 							 key[5];
		  
	offset = (BYTE) FindComponentOffset(componentList, 9);
	ccode =  GetMachineStaticInfo(connectionID, offset, &intvptr, &Statdata);

	curportal = CreatePortal(
    	/* topline         */ 3,
    	/* leftColumn      */ 5,
    	/* frameHeight     */ 22,
    	/* frameWidth      */ 65,
    	/* virtualHeight   */ 22,
    	/* virtualWidth    */ 65,
    	/* saveFlag        */ SAVE,
    	/* headerText      */ GetMessage(WSSTATIC_STATS_DISPLAY),
    	/* headerAttr      */ VNORMAL,
    	/* borderType      */ SINGLE,
    	/* borderAttr      */ VINTENSE,
    	/* cursorState     */ CURSOR_OFF,
    	/* directFlag      */ VIRTUAL
    	);

   oldportal=DeselectPortal();
   SelectPortal(curportal);

   do
		{

   	ClearPortal();

   	DisplayTextInPortal(0,5,"Start Date               : ",VNORMAL);
   	DisplayTextInPortal(1,5,"Start Time               : ",VNORMAL);
   	DisplayTextInPortal(2,5,"OS Version Major         : ",VNORMAL);
   	DisplayTextInPortal(3,5,"OS Version Minor         : ",VNORMAL);
   	DisplayTextInPortal(4,5,"OS Version Type          : ",VNORMAL);
   	DisplayTextInPortal(5,5,"Number of Serial Ports   : ",VNORMAL);
   	DisplayTextInPortal(6,5,"Number of Parallel Ports : ",VNORMAL);
   	DisplayTextInPortal(7,5,"Floppy Drive Count       : ",VNORMAL);
   	DisplayTextInPortal(8,5,"Floppy Drive Type        : ",VNORMAL);
   	DisplayTextInPortal(9,5,"Hard Drive Count         : ",VNORMAL);
   	DisplayTextInPortal(10,5,"Key Board Type           : ",VNORMAL);
   	DisplayTextInPortal(11,5,"Boot Video Type          : ",VNORMAL);
   	DisplayTextInPortal(12,5,"Display Memory           :",VNORMAL);
   	DisplayTextInPortal(13,5,"Math Coprocessor         : ",VNORMAL);
   	DisplayTextInPortal(14,5,"Coprocessor Type         :",VNORMAL);
   	DisplayTextInPortal(15,5,"CPU Type                 :",VNORMAL);
   	DisplayTextInPortal(16,5,"Model Type               :",VNORMAL);
   	DisplayTextInPortal(17,5,"Serial Address           :",VNORMAL);
	
   	sprintf(txtbuff, "%LX", Statdata.StartDate);
   	DisplayTextInPortal(0,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%LX",Statdata.StartTime);
   	DisplayTextInPortal(1,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.OSVerMajor);
   	DisplayTextInPortal(2,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.OSVerMinor);
   	DisplayTextInPortal(3,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.OSVersionType);
   	DisplayTextInPortal(4,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u", Statdata.NumberOfSerialPorts);
   	DisplayTextInPortal(5,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.NumberOfParallelPorts);
   	DisplayTextInPortal(6,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.FloppyDriveCount);
   	DisplayTextInPortal(7,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.FloppyDriveType);
   	DisplayTextInPortal(8,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.HardDriveCount);
   	DisplayTextInPortal(9,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.KeyboardType);
   	DisplayTextInPortal(10,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.BootVideoType);
   	DisplayTextInPortal(11,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%Lu",Statdata.DisplayMemory);
   	DisplayTextInPortal(12,35,txtbuff,VNORMAL);

		sprintf(txtbuff, "%u",Statdata.MathCoprocessor);
   	DisplayTextInPortal(13,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.CoprocessorType);
   	DisplayTextInPortal(14,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.CPUType);
   	DisplayTextInPortal(15,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.ModelType);
   	DisplayTextInPortal(16,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%03X:%03X:%03X:%03X",
			Statdata.SerialAddress[0],
			Statdata.SerialAddress[1],
			Statdata.SerialAddress[2],
			Statdata.SerialAddress[3]
			);
   	DisplayTextInPortal(17,35,txtbuff,VNORMAL);

   	UpdatePortal();
		WaitForAnyKey();
		ClearPortal();

   	DisplayTextInPortal(0,5,"Parallel Address         : ",VNORMAL);
   	DisplayTextInPortal(1,5,"Pointer Device           : ",VNORMAL);
   	DisplayTextInPortal(2,5,"System Memory            : ",VNORMAL);
   	DisplayTextInPortal(3,5,"Memory Types             : ",VNORMAL);
   	DisplayTextInPortal(4,5,"BIOS Type                : ",VNORMAL);
   	DisplayTextInPortal(5,5,"BIOS Revision            : ",VNORMAL);
   	DisplayTextInPortal(6,5,"EISA MCA Slots           : ",VNORMAL);
   	DisplayTextInPortal(7,5,"EISA MCA IRQ             : ",VNORMAL);
   	DisplayTextInPortal(8,5,"EISA Card Name           : ",VNORMAL);
   	DisplayTextInPortal(9,5,"Bus Type                 : ",VNORMAL);
   	DisplayTextInPortal(10,5,"Hard Drive Type          : ",VNORMAL);
   	DisplayTextInPortal(11,5,"Memory Type Flag         : ",VNORMAL);
   	DisplayTextInPortal(12,5,"Total XMS                :",VNORMAL);
   	DisplayTextInPortal(13,5,"Available XMS            : ",VNORMAL);
   	DisplayTextInPortal(14,5,"Total EMS                :",VNORMAL);
   	DisplayTextInPortal(15,5,"Available EMS            :",VNORMAL);


   	sprintf(txtbuff, "%03X:%03X:%03X:%03X",
			Statdata.ParallelAddress[0],
			Statdata.ParallelAddress[1],
			Statdata.ParallelAddress[2],
			Statdata.ParallelAddress[3]
			);

   	DisplayTextInPortal(0,35,txtbuff,VNORMAL);


   	sprintf(txtbuff, "%u",Statdata.PointerDevice);
   	DisplayTextInPortal(1,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.SystemMemory);
   	DisplayTextInPortal(2,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.MemoryTypes);
   	DisplayTextInPortal(3,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.BIOSType);
   	DisplayTextInPortal(4,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u:%u", Statdata.BIOSRev[0], Statdata.BIOSRev[1]);
   	DisplayTextInPortal(5,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.EISA_MCASlots);
   	DisplayTextInPortal(6,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.EISA_MCAIRQ[0]);
   	DisplayTextInPortal(7,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.EISA_CardName);
   	DisplayTextInPortal(8,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.BusType);
   	DisplayTextInPortal(9,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u:%u:%u:%u:%u",Statdata.HardDriveType[0],
		                                	Statdata.HardDriveType[1],
												  	Statdata.HardDriveType[2],
												  	Statdata.HardDriveType[3],
												  	Statdata.HardDriveType[4]);

   	DisplayTextInPortal(10,35,txtbuff,VNORMAL);


   	sprintf(txtbuff, "%u",Statdata.MemTypeFlag);
   	DisplayTextInPortal(11,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.TotalXMS);
   	DisplayTextInPortal(12,35,txtbuff,VNORMAL);

		sprintf(txtbuff, "%u",Statdata.AvailableXMS);
   	DisplayTextInPortal(13,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.TotalEMS);
   	DisplayTextInPortal(14,35,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",Statdata.AvailableEMS);
   	DisplayTextInPortal(15,35,txtbuff,VNORMAL);

   	UpdatePortal();

		kstat = WaitForFunctionKey(key);

   	} while(*(key) != I_ESC);

	ReselectPortal(oldportal);
	DestroyPortal(curportal);
		

}


/**************************************************************************
** Generic Network Management Agent Menu Action Procedure
*/

int GNMAMenuAct(int operation, LIST **element)
	{
	
	switch(operation)
		{
		case -1:
		   return(0);
		   break;

		case WSSTATIC_STATS:
		   WSStaticStat();
		   break;

		case PROTOCOL_INFO:
		   ProtocolInfo();
		   break;

		case MLID_INFO:
		   MLIDInfo();
		   break;

		}

   return(-1);
	}


/**************************************************************************
** Generic Network Management Agent Menu Action Procedure
*/

void GNMAmenu()
	{

   PushList();
   InitList();
   AppendToPopUpMenu(WSSTATIC_STATS, WSSTATIC_STATS, NO_HELP_CONTEXT);
   AppendToPopUpMenu(PROTOCOL_INFO, PROTOCOL_INFO, NO_HELP_CONTEXT);
   AppendToPopUpMenu(MLID_INFO, MLID_INFO, NO_HELP_CONTEXT);

   PopUpMenu(
		/* MenuHeader       */ GNMA_MENU_HDR,
		/* centerLine y     */ 0,
		/* centerColumn x   */ 0,
		/* menuborder       */ DOUBLE,
		/* startOption      */ NullPointer(LIST),
		/* menuFlags        */ MENU_NORMAL,
		/* menuHelp         */ NO_HELP_CONTEXT,
		/* action procedure */ GNMAMenuAct
		);

   DestroyList();
   PopList();


	}

/**************************************************************************
** Builds and displays Address Table / Connection ID Table
*/

void ADTstat(void)
	{
	int							 kstat;
	int							 ccode;
	int							 count;
	int                      curportal;
	int                      oldportal;
	BYTE                     tmpbuff[10];
	BYTE                     offset;
	AllResponseData          intvptr;
	ServerAddressTableStruct Atdata;
	ServerNameTableStruct	 Stdata;
	char                     txtbuff[90];
	char 							 key[5];
	char							 nusedbuff[48]="Connection Table Entry Not Used!";
					
		
	offset = (BYTE) FindComponentOffset(componentList, 3);
	ccode  = GetServerAddressTable (connectionID, offset, &intvptr, &Atdata);
	offset = (BYTE) FindComponentOffset(componentList, 3);
	ccode  = GetServerNameTable (connectionID, offset, &intvptr, &Stdata);
	
	PushHelpContext(CONNECTION_TABLE_HELP);

	curportal = CreatePortal(
   	/* topline         */ 5,
   	/* leftColumn      */ 5,
   	/* frameHeight     */ 18,
   	/* frameWidth      */ 65,
   	/* virtualHeight   */ 18,
   	/* virtualWidth    */ 65,
   	/* saveFlag        */ SAVE,
   	/* headerText      */ GetMessage(CONN_TABLE_DISPLAY),
   	/* headerAttr      */ VNORMAL,
   	/* borderType      */ SINGLE,
   	/* borderAttr      */ VINTENSE,
   	/* cursorState     */ CURSOR_OFF,
   	/* directFlag      */ VIRTUAL
   	);
   oldportal=DeselectPortal();
   SelectPortal(curportal);
   ClearPortal();

	count=0;
   do
		{
   	DisplayTextInPortal(0,2, "Server Name            : ",VNORMAL);
   	DisplayTextInPortal(1,2, "Server Used            : ",VNORMAL);
   	DisplayTextInPortal(2,2, "Order Number           : ",VNORMAL);
   	DisplayTextInPortal(3,2, "Server Network Address : ",VNORMAL);
   	DisplayTextInPortal(4,2, "Server Node Address    : ",VNORMAL);
   	DisplayTextInPortal(5,2, "Server Socket          : ",VNORMAL);
   	DisplayTextInPortal(6,2, "Received Time Out      : ",VNORMAL);
   	DisplayTextInPortal(7,2, "Immediate Node         : ",VNORMAL);
   	DisplayTextInPortal(8,2, "Sequence Number        : ",VNORMAL);
   	DisplayTextInPortal(9,2, "Connection Number      : ",VNORMAL);
   	DisplayTextInPortal(10,2, "Connection OK          : ",VNORMAL);
   	DisplayTextInPortal(11,2, "Maximum Time Out       : ",VNORMAL);
   	DisplayTextInPortal(12,2, "Connection Entry Number: ",VNORMAL);


		if(Atdata.addressTable[count].serverUsed != 0x00)
	 		{
    		sprintf(txtbuff, "%s", Stdata.nameTable[count].name);
    		DisplayTextInPortal(0,28,txtbuff,VNORMAL);
    		}
		else
	 		{
    		sprintf(txtbuff, "%s", nusedbuff);
    		DisplayTextInPortal(0,28,txtbuff,VNORMAL);
    		}

   	sprintf(txtbuff, "%02X", Atdata.addressTable[count].serverUsed);
   	DisplayTextInPortal(1,28,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%02X",Atdata.addressTable[count].orderNumber);
   	DisplayTextInPortal(2,28,txtbuff,VNORMAL);

   	sprintf(tmpbuff, "%02X%02X%02X%02X",
			Atdata.addressTable[count].serverNetwork[0],
			Atdata.addressTable[count].serverNetwork[1],
			Atdata.addressTable[count].serverNetwork[2],
			Atdata.addressTable[count].serverNetwork[3]);
   	DisplayTextInPortal(3,28,tmpbuff,VNORMAL);

   	sprintf(txtbuff, "%02X%02X%02X%02X%02X%02X",
			Atdata.addressTable[count].serverNode[0],
			Atdata.addressTable[count].serverNode[1],
			Atdata.addressTable[count].serverNode[2],
			Atdata.addressTable[count].serverNode[3],
			Atdata.addressTable[count].serverNode[4],
			Atdata.addressTable[count].serverNode[5]);
   	DisplayTextInPortal(4,28,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%04X",IntSwap(Atdata.addressTable[count].serverSocket));
   	DisplayTextInPortal(5,28,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%d",Atdata.addressTable[count].receivedTimeOut);
   	DisplayTextInPortal(6,28,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%02X%02X%02X%02X%02X%02X",
			Atdata.addressTable[count].immediateNode[0],
			Atdata.addressTable[count].immediateNode[1],
			Atdata.addressTable[count].immediateNode[2],
			Atdata.addressTable[count].immediateNode[3],
			Atdata.addressTable[count].immediateNode[4],
			Atdata.addressTable[count].immediateNode[5]);
   	DisplayTextInPortal(7,28,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%d",Atdata.addressTable[count].sequenceNumber);
   	DisplayTextInPortal(8,28,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%d",Atdata.addressTable[count].connectionNumber);
   	DisplayTextInPortal(9,28,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%d",Atdata.addressTable[count].connectionOK);
   	DisplayTextInPortal(10,28,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%d",Atdata.addressTable[count].maximumTimeOut);
   	DisplayTextInPortal(11,28,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%d",count);
   	DisplayTextInPortal(12,28,txtbuff,VNORMAL);

		sprintf(txtbuff, "                       ");
   	DisplayTextInPortal(13,38,txtbuff,VNORMAL);

		if(count <= 0)
	 		{
	 		sprintf(txtbuff, "PAGE UP or UP ARROW");
    		DisplayTextInPortal(13,38,txtbuff,VNORMAL);
    		}
		else if(count >= 7)
	 		{
	 		sprintf(txtbuff, "PAGE DOWN or DOWN ARROW");
    		DisplayTextInPortal(13,38,txtbuff,VNORMAL);
	 		}

   	UpdatePortal();

		kstat = WaitForFunctionKey(key);
      ClearPortal();


		if((*(key) == I_PAGE_UP) || (*(key) == I_UP))
			{
			count = count + 1;
			if(count >= 7)
		   	count = 7;
			}

		if((*(key) == I_PAGE_DOWN) || (*(key) == I_DOWN))
			{
			count = count - 1;
			if(count <= 0)
				count = 0;
			}
    
   	} while(*(key) != I_ESC);

	PopHelpContext();
	ReselectPortal(oldportal);
	DestroyPortal(curportal);

	return;
	}

/**************************************************************************
** Display Workstation Statistics
*/
void WsStat(void)
	{
	int                             curportal;
	int                             oldportal;
	char                            txtbuff[90];
	BYTE                            tmpbuff[10];
	int                             ccode;
	AllResponseData                 intvptr;
	OSVersionStruct                 OSdata;
	ShellAddressStruct              Addata;
	DriverConfigurationStruct       Drdata;
	ShellVersionStruct				  Svdata;
	BYTE                            offset;

	PushHelpContext(WS_STATS_HELP);

	offset = (BYTE) FindComponentOffset(componentList, 3);
	ccode = GetOSVersionInfo (connectionID, offset, &intvptr, &OSdata);
	ccode = GetShellAddress (connectionID, offset, &intvptr, &Addata);
	offset = (BYTE) FindComponentOffset(componentList, 2);
	ccode = GetShellDriverConfiguration (connectionID,	offset, &intvptr, &Drdata);
	offset = (BYTE) FindComponentOffset(componentList, 3);
	ccode = GetShellVersionInfo (connectionID, offset, &intvptr, &Svdata);
	
	curportal = CreatePortal(
   	/* topline         */ 5,
   	/* leftColumn      */ 5,
   	/* frameHeight     */ 11,
   	/* frameWidth      */ 65,
   	/* virtualHeight   */ 11,
   	/* virtualWidth    */ 65,
   	/* saveFlag        */ SAVE,
   	/* headerText      */ GetMessage(WS_INFO_DISPLAY),
   	/* headerAttr      */ VNORMAL,
   	/* borderType      */ SINGLE,
   	/* borderAttr      */ VINTENSE,
   	/* cursorState     */ CURSOR_OFF,
   	/* directFlag      */ VIRTUAL
   	);
   oldportal=DeselectPortal();
   SelectPortal(curportal);
   ClearPortal();


   DisplayTextInPortal(0,2,"LAN Board Settings : ",VNORMAL);
   DisplayTextInPortal(1,2,"Operating System   : ",VNORMAL);
   DisplayTextInPortal(2,2,"Network Address    : ",VNORMAL);
   DisplayTextInPortal(3,2,"Local Node Address : ",VNORMAL);
   DisplayTextInPortal(4,2,"Local Socket       : ",VNORMAL);
   DisplayTextInPortal(5,2,"Shell Version      : ",VNORMAL);
	
   sprintf(txtbuff, "%s", Drdata.textDescription);
   DisplayTextInPortal(0,23,txtbuff,VNORMAL);
   sprintf(txtbuff, "%s",OSdata.versionData);
   DisplayTextInPortal(1,23,txtbuff,VNORMAL);
   sprintf(tmpbuff, "%02X%02X%02X%02X",
		Addata.network[0],
		Addata.network[1],
		Addata.network[2],
		Addata.network[3]
		);
   DisplayTextInPortal(2,23,tmpbuff,VNORMAL);
   sprintf(txtbuff, "%02X%02X%02X%02X%02X%02X",
		Addata.node[0],
		Addata.node[1],
		Addata.node[2],
		Addata.node[3],
		Addata.node[4],
		Addata.node[5]
		);
   DisplayTextInPortal(3,23,txtbuff,VNORMAL);

   sprintf(txtbuff, "%04X",Addata.socket);
   DisplayTextInPortal(4,23,txtbuff,VNORMAL);

   sprintf(txtbuff, "v%d.%d%x",Svdata.major, Svdata.minor, Svdata.rev);
   DisplayTextInPortal(5,23,txtbuff,VNORMAL);

   UpdatePortal();
   WaitForEscape();
	PopHelpContext();
	ReselectPortal(oldportal);
	DestroyPortal(curportal);
	
	return;
	}

/**************************************************************************
** Display Shell Statistics
*/
void ShellStat(void)
	{
	int                     curportal;
	int                     oldportal;
	char                    txtbuff[80];
	int                     ccode;
	AllResponseData         intvptr;
	ShellStatisticsStruct   Sdata;
	BYTE                    offset;

	offset = (BYTE) FindComponentOffset(componentList, 3);
	ccode = GetShellStatistics (connectionID, offset, &intvptr, &Sdata);

	PushHelpContext(SHELL_STATS_HELP);

	curportal = CreatePortal(
   	/* topline         */ 3,
   	/* leftColumn      */ 5,
   	/* frameHeight     */ 21,
   	/* frameWidth      */ 65,
   	/* virtualHeight   */ 21,
   	/* virtualWidth    */ 65,
   	/* saveFlag        */ SAVE,
   	/* headerText      */ GetMessage(SHELL_DRV_DISPLAY),
   	/* headerAttr      */ VNORMAL,
   	/* borderType      */ SINGLE,
   	/* borderAttr      */ VINTENSE,
   	/* cursorState     */ CURSOR_OFF,
   	/* directFlag      */ VIRTUAL
   	);
   oldportal=DeselectPortal();
   SelectPortal(curportal);
   ClearPortal();

   DisplayTextInPortal(0,5,"Shell Requests Count                : ",VNORMAL);
   DisplayTextInPortal(1,5,"Operator Aborts Count               : ",VNORMAL);
   DisplayTextInPortal(2,5,"Operator Retries Count              : ",VNORMAL);
   DisplayTextInPortal(3,5,"Time Outs Count                     : ",VNORMAL);
   DisplayTextInPortal(4,5,"Write Error Count                   : ",VNORMAL);
   DisplayTextInPortal(5,5,"Invalid Reply Header Count          : ",VNORMAL);
   DisplayTextInPortal(6,5,"Invalid Slot Count                  : ",VNORMAL);
   DisplayTextInPortal(7,5,"Invalid Sequence Number Count       : ",VNORMAL);
   DisplayTextInPortal(8,5,"Error Receiving Count               : ",VNORMAL);
   DisplayTextInPortal(9,5,"No Router Found Count               : ",VNORMAL);
   DisplayTextInPortal(10,5,"Being Processed Count               : ",VNORMAL);
   DisplayTextInPortal(11,5,"Unknown Error Count                 : ",VNORMAL);
   DisplayTextInPortal(12,5,"Invalid Server Slot Count           :",VNORMAL);
   DisplayTextInPortal(13,5,"NetWork Gone Count                  : ",VNORMAL);
   DisplayTextInPortal(14,5,"Allocate Cannot Find Route Count    :",VNORMAL);
   DisplayTextInPortal(15,5,"Allocate No Slots Available Count   :",VNORMAL);
   DisplayTextInPortal(16,5,"Allocate Server is Down Count       :",VNORMAL);
	
   sprintf(txtbuff, "%ld", Sdata.shellRequestsCount);
   DisplayTextInPortal(0,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.operatorAbortsCount);
   DisplayTextInPortal(1,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.operatorRetriesCount);
   DisplayTextInPortal(2,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.timeoutsCount);
   DisplayTextInPortal(3,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u", Sdata.writeErrorCount);
   DisplayTextInPortal(4,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.invalidReplyHeaderCount);
   DisplayTextInPortal(5,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.invalidSlotCount);
   DisplayTextInPortal(6,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.invalidSequenceNumberCount);
   DisplayTextInPortal(7,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.errorReceivingCount);
   DisplayTextInPortal(8,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.noRouterFoundCount);
   DisplayTextInPortal(9,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.beingProcessedCount);
   DisplayTextInPortal(10,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.unknownErrorCount);
   DisplayTextInPortal(11,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.invalidServerSlotCount);
   DisplayTextInPortal(12,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.networkGoneCount);
   DisplayTextInPortal(13,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.allocateCannotFindRouteCount);
   DisplayTextInPortal(14,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.allocateNoSlotsAvailableCount);
   DisplayTextInPortal(15,45,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",Sdata.allocateServerIsDownCount);
   DisplayTextInPortal(16,45,txtbuff,VNORMAL);

   UpdatePortal();
   WaitForEscape();
	PopHelpContext();
	ReselectPortal(oldportal);
	DestroyPortal(curportal);
	
	return;
	}

/****************************************************************************
**	SPX Statistics
*/
void SPXStat(void)
	{
	int                     curportal;
	int                     oldportal;
	char                    txtbuff[80];
	int                     ccode;
	AllResponseData         intvptr;
	SPXStatisticsStruct     SPXdata;
	BYTE                    offset;

	offset = (BYTE) FindComponentOffset(componentList, 0);
	ccode = GetSPXStatistics (connectionID, offset, &intvptr, &SPXdata);

	PushHelpContext(SPX_STATS_HELP);

	curportal = CreatePortal(
    	/* topline         */ 3,
    	/* leftColumn      */ 5,
    	/* frameHeight     */ 22,
    	/* frameWidth      */ 65,
    	/* virtualHeight   */ 22,
    	/* virtualWidth    */ 65,
    	/* saveFlag        */ SAVE,
    	/* headerText      */ GetMessage(SPX_STATS_DISPLAY),
    	/* headerAttr      */ VNORMAL,
    	/* borderType      */ SINGLE,
    	/* borderAttr      */ VINTENSE,
    	/* cursorState     */ CURSOR_OFF,
    	/* directFlag      */ VIRTUAL
    	);
    oldportal=DeselectPortal();
    SelectPortal(curportal);
    ClearPortal();

    DisplayTextInPortal(0,5,"Max Connections Count           : ",VNORMAL);
    DisplayTextInPortal(1,5,"Max Used Connections Count      : ",VNORMAL);
    DisplayTextInPortal(2,5,"Establish Connection Request    : ",VNORMAL);
    DisplayTextInPortal(3,5,"Establisch Connection Failure   : ",VNORMAL);
    DisplayTextInPortal(4,5,"Listen Connection Request Count : ",VNORMAL);
    DisplayTextInPortal(5,5,"Listen Connection Failure Count : ",VNORMAL);
    DisplayTextInPortal(6,5,"Send Packet Count               : ",VNORMAL);
    DisplayTextInPortal(7,5,"Window Choke Count              : ",VNORMAL);
    DisplayTextInPortal(8,5,"Bad Send Packet Count           : ",VNORMAL);
    DisplayTextInPortal(9,5,"Send Failure Count              : ",VNORMAL);
    DisplayTextInPortal(10,5,"Abort Connection Count          : ",VNORMAL);
    DisplayTextInPortal(11,5,"Listen Packet Count             : ",VNORMAL);
    DisplayTextInPortal(12,5,"Bad Listen Packet Count         :",VNORMAL);
    DisplayTextInPortal(13,5,"Incoming Packet Count           : ",VNORMAL);
    DisplayTextInPortal(14,5,"Bad Incoming Packet Count       :",VNORMAL);
    DisplayTextInPortal(15,5,"Suppressed Packet Count         :",VNORMAL);
    DisplayTextInPortal(16,5,"No Session Listen ECB Count     :",VNORMAL);
    DisplayTextInPortal(17,5,"Watchdog Destroy Session Count  :",VNORMAL);
	
    sprintf(txtbuff, "%u", SPXdata.maxConnectionsCount);
    DisplayTextInPortal(0,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.maxUsedConnectionsCount);
    DisplayTextInPortal(1,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.establishConnectionRequest);
    DisplayTextInPortal(2,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.establishConnectionFailure);
    DisplayTextInPortal(3,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.listenConnectionRequestCount);
    DisplayTextInPortal(4,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u", SPXdata.listenConnectionFailureCount);
    DisplayTextInPortal(5,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%ld",SPXdata.sendPacketCount);
    DisplayTextInPortal(6,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%ld",SPXdata.windowChokeCount);
    DisplayTextInPortal(7,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.badSendPacketCount);
    DisplayTextInPortal(8,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.sendFailureCount);
    DisplayTextInPortal(9,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.abortConnectionCount);
    DisplayTextInPortal(10,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%ld",SPXdata.listenPacketCount);
    DisplayTextInPortal(11,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.badListenPacketCount);
    DisplayTextInPortal(12,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%ld",SPXdata.incomingPacketCount);
    DisplayTextInPortal(13,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.badIncomingPacketCount);
    DisplayTextInPortal(14,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.suppressedPacketCount);
    DisplayTextInPortal(15,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.noSessionListenECBCount);
    DisplayTextInPortal(16,45,txtbuff,VNORMAL);
    sprintf(txtbuff, "%u",SPXdata.watchdogDestroySessionCount);
    DisplayTextInPortal(17,45,txtbuff,VNORMAL);

    UpdatePortal();
    WaitForEscape();
	PopHelpContext();
	ReselectPortal(oldportal);
	DestroyPortal(curportal);
		
	return;
	}

/***************************************************************************
** Builds and displays Shell Driver Configuration (returns LAN Driver conf.)
*/
void LANConfig(void)
{
	int							 kstat;
	int                      curportal;
	int                      oldportal;
	int							 ccode;
	BYTE                     offset;
	char                     txtbuff[80];
	DriverConfigurationStruct	 MLIDdata;
	AllResponseData          intvptr;
	char 							 key[5];
		  
	offset = (BYTE) FindComponentOffset(componentList, 2);
	ccode =  GetShellDriverConfiguration(connectionID, offset, &intvptr, &MLIDdata);

	PushHelpContext(LAN_CONFIG_HELP);

	curportal = CreatePortal(
    	/* topline         */ 3,
    	/* leftColumn      */ 5,
    	/* frameHeight     */ 22,
    	/* frameWidth      */ 65,
    	/* virtualHeight   */ 22,
    	/* virtualWidth    */ 65,
    	/* saveFlag        */ SAVE,
    	/* headerText      */ GetMessage(MLID_STATS_DISPLAY),
    	/* headerAttr      */ VNORMAL,
    	/* borderType      */ SINGLE,
    	/* borderAttr      */ VINTENSE,
    	/* cursorState     */ CURSOR_OFF,
    	/* directFlag      */ VIRTUAL
    	);

   oldportal=DeselectPortal();
   SelectPortal(curportal);

   do
		{

   	ClearPortal();

   	DisplayTextInPortal(0,5,"NetWork Address       : ",VNORMAL);
   	DisplayTextInPortal(1,5,"Node Address          : ",VNORMAL);
   	DisplayTextInPortal(2,5,"Node Address Type     : ",VNORMAL);
   	DisplayTextInPortal(3,5,"Maximum Data Size     : ",VNORMAL);
   	DisplayTextInPortal(4,5,"LAN Hardware ID       : ",VNORMAL);
   	DisplayTextInPortal(5,5,"Transport Time        : ",VNORMAL);
   	DisplayTextInPortal(6,5,"Major Version         : ",VNORMAL);
   	DisplayTextInPortal(7,5,"Minor Version         : ",VNORMAL);
   	DisplayTextInPortal(8,5,"LAN Description       : ",VNORMAL);
   	DisplayTextInPortal(9,5,"IO Address #1         : ",VNORMAL);
   	DisplayTextInPortal(10,5,"IO Decode Range #1    : ",VNORMAL);
   	DisplayTextInPortal(11,5,"IO Address #2         : ",VNORMAL);
   	DisplayTextInPortal(12,5,"IO Decode Range #2    : ",VNORMAL);
   	DisplayTextInPortal(13,5,"Memory Address #1     : ",VNORMAL);
   	DisplayTextInPortal(14,5,"Memory Decode Range1  : ",VNORMAL);
   	DisplayTextInPortal(15,5,"Memory Address #2     : ",VNORMAL);
   	DisplayTextInPortal(16,5,"Memory Decode Range2  : ",VNORMAL);
	
   	sprintf(txtbuff, "%02X%02X%02X%02X",
			MLIDdata.networkAddress[0],
		   MLIDdata.networkAddress[1],	
		   MLIDdata.networkAddress[2],	
		   MLIDdata.networkAddress[3]);	

   	DisplayTextInPortal(0,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%02X%02X%02X%02X%02X%02X",
			MLIDdata.nodeAddress[0],
		   MLIDdata.nodeAddress[1],	
		   MLIDdata.nodeAddress[2],	
		   MLIDdata.nodeAddress[3],	
		   MLIDdata.nodeAddress[4],	
			MLIDdata.nodeAddress[5]);

   	DisplayTextInPortal(1,30,txtbuff,VNORMAL);


   	sprintf(txtbuff, "%02X",MLIDdata.nodeAddressType);
   	DisplayTextInPortal(2,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.maxDataSize);
   	DisplayTextInPortal(3,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.reserved4);
   	DisplayTextInPortal(4,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u", MLIDdata.transportTime);
   	DisplayTextInPortal(5,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.majorVersion);
   	DisplayTextInPortal(6,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.minorVersion);
   	DisplayTextInPortal(7,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%s",MLIDdata.LANDescription);
   	DisplayTextInPortal(8,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%X",MLIDdata.IOAddress1);
   	DisplayTextInPortal(9,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%X",MLIDdata.IODecodeRange1);
   	DisplayTextInPortal(10,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%X",MLIDdata.IOAddress2);
   	DisplayTextInPortal(11,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%X",MLIDdata.IODecodeRange2);
   	DisplayTextInPortal(12,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%02X%02X:%02X",
			MLIDdata.memoryAddress1[2],
		   MLIDdata.memoryAddress1[1],	
		   MLIDdata.memoryAddress1[0]);	

   	DisplayTextInPortal(13,30,txtbuff,VNORMAL);


   	sprintf(txtbuff, "%04X",MLIDdata.memoryDecodeRange1);
   	DisplayTextInPortal(14,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%02X%02X:%02X",
			MLIDdata.memoryAddress2[2],
		   MLIDdata.memoryAddress2[1],	
		   MLIDdata.memoryAddress2[0]);	

   	DisplayTextInPortal(15,30,txtbuff,VNORMAL);


   	sprintf(txtbuff, "%04X",MLIDdata.memoryDecodeRange2);
   	DisplayTextInPortal(16,30,txtbuff,VNORMAL);



   	UpdatePortal();
		WaitForAnyKey();
		ClearPortal();

   	DisplayTextInPortal(0,5,"Interrupt is Used #1  : ",VNORMAL);
   	DisplayTextInPortal(1,5,"Interrupt Line #1     : ",VNORMAL);
   	DisplayTextInPortal(2,5,"Interrupt is Used #2  : ",VNORMAL);
   	DisplayTextInPortal(3,5,"Interrupt Line #2     : ",VNORMAL);
   	DisplayTextInPortal(4,5,"DMA is Used #1        : ",VNORMAL);
   	DisplayTextInPortal(5,5,"DMA Line #1           : ",VNORMAL);
   	DisplayTextInPortal(6,5,"DMA is Used #2        : ",VNORMAL);
   	DisplayTextInPortal(7,5,"DMA Line #2           : ",VNORMAL);
   	DisplayTextInPortal(8,5,"Current LAN Settings  : ",VNORMAL);

   	sprintf(txtbuff, "%X",MLIDdata.interruptIsUsed1);
   	DisplayTextInPortal(0,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.interruptLine1);
   	DisplayTextInPortal(1,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%X",MLIDdata.interruptIsUsed2);
   	DisplayTextInPortal(2,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.interruptLine2);
   	DisplayTextInPortal(3,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%X",MLIDdata.DMAIsUsed1);
   	DisplayTextInPortal(4,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u", MLIDdata.DMALine1);
   	DisplayTextInPortal(5,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%X",MLIDdata.DMAIsUsed2);
   	DisplayTextInPortal(6,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%u",MLIDdata.DMALine2);
   	DisplayTextInPortal(7,30,txtbuff,VNORMAL);

   	sprintf(txtbuff, "%s",MLIDdata.textDescription);
   	DisplayTextInPortal(8,30,txtbuff,VNORMAL);


   	UpdatePortal();

		kstat = WaitForFunctionKey(key);

   	} while(*(key) != I_ESC);

	PopHelpContext();
	ReselectPortal(oldportal);
	DestroyPortal(curportal);
		

}


/***************************************************************************
** Builds and displays Shell Driver statistics (returns LAN Driver stats)
*/
void LANStat(void)
	{
	int                     curportal;
	int                     oldportal;
	char                    txtbuff[80];
	int                     ccode;
	AllResponseData         intvptr;
	DriverStatisticsStruct  LANdata;
	BYTE                    offset;

	offset = (BYTE) FindComponentOffset(componentList, 2);
	ccode = GetShellDriverStatistics (connectionID, offset, &intvptr, &LANdata);

	PushHelpContext(LAN_STATS_HELP);

	curportal = CreatePortal(
   	/* topline         */ 5,
   	/* leftColumn      */ 5,
   	/* frameHeight     */ 18,
   	/* frameWidth      */ 65,
   	/* virtualHeight   */ 18,
   	/* virtualWidth    */ 65,
   	/* saveFlag        */ SAVE,
   	/* headerText      */ GetMessage(LAN_DRV_DISPLAY),
   	/* headerAttr      */ VNORMAL,
   	/* borderType      */ SINGLE,
   	/* borderAttr      */ VINTENSE,
   	/* cursorState     */ CURSOR_OFF,
   	/* directFlag      */ VIRTUAL
   	);
   oldportal=DeselectPortal();
   SelectPortal(curportal);
   ClearPortal();

   DisplayTextInPortal(0,5,"Total TX Packet Count       : ",VNORMAL);
   DisplayTextInPortal(1,5,"Total RX Packet Count       : ",VNORMAL);
   DisplayTextInPortal(2,5,"No ECB Available Count      : ",VNORMAL);
   DisplayTextInPortal(3,5,"Packet TX Too Big Count     : ",VNORMAL);
   DisplayTextInPortal(4,5,"Packet TX Too Small Count   : ",VNORMAL);
   DisplayTextInPortal(5,5,"Packet TX Overflow Count    : ",VNORMAL);
   DisplayTextInPortal(6,5,"Packet RX Too Big Count     : ",VNORMAL);
   DisplayTextInPortal(7,5,"Packet RX Too Small Count   : ",VNORMAL);
   DisplayTextInPortal(8,5,"Packet TX Misc Error Count  : ",VNORMAL);
   DisplayTextInPortal(9,5,"Packet RX Misc Error Count  : ",VNORMAL);
   DisplayTextInPortal(10,5,"Retry TX count              : ",VNORMAL);
   DisplayTextInPortal(11,5,"Checksum Error Count        : ",VNORMAL);
   DisplayTextInPortal(12,5,"Hardware RX Mismatch Count  : ",VNORMAL);
	
   sprintf(txtbuff, "%ld", LANdata.totalTxPacketCount);
   DisplayTextInPortal(0,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%ld",LANdata.totalRxPacketCount);
   DisplayTextInPortal(1,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",LANdata.noECBAvailableCount);
   DisplayTextInPortal(2,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",LANdata.packetTxTooBigCount);
   DisplayTextInPortal(3,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",LANdata.packetTxTooSmallCount);
   DisplayTextInPortal(4,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u", LANdata.packetRxOverflowCount);
   DisplayTextInPortal(5,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",LANdata.packetRxTooBigCount);
   DisplayTextInPortal(6,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",LANdata.packetRxTooSmallCount);
   DisplayTextInPortal(7,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",LANdata.packetTxMiscErrorCount);
   DisplayTextInPortal(8,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",LANdata.packetRxMiscErrorCount);
   DisplayTextInPortal(9,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",LANdata.retryTxCount);
   DisplayTextInPortal(10,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",LANdata.checksumErrorCount);
   DisplayTextInPortal(11,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",LANdata.hardwareRxMismatchCount);
   DisplayTextInPortal(12,35,txtbuff,VNORMAL);

   UpdatePortal();
   WaitForEscape();

	PopHelpContext();
	ReselectPortal(oldportal);
	DestroyPortal(curportal);
	
	return;
	}

/**************************************************************************
** Display IPX Statistics
*/
void IpxStat(void)
	{
	int                             curportal;
	int                             oldportal;
	char                            txtbuff[80];
	int                             ccode;
	AllResponseData                 intvptr;
	IPXStatisticsStruct             IPXdata;
	BYTE                            offset;

	offset = (BYTE) FindComponentOffset(componentList, 0);
	ccode = GetIPXStatistics (connectionID, offset, &intvptr, &IPXdata);

	PushHelpContext(IPX_STATS_HELP);

	curportal = CreatePortal(
   /* topline         */ 5,
   /* leftColumn      */ 5,
   /* frameHeight     */ 16,
   /* frameWidth      */ 65,
   /* virtualHeight   */ 16,
   /* virtualWidth    */ 65,
   /* saveFlag        */ SAVE,
   /* headerText      */ GetMessage(IPX_STATS_DISPLAY),
   /* headerAttr      */ VNORMAL,
   /* borderType      */ SINGLE,
   /* borderAttr      */ VINTENSE,
   /* cursorState     */ CURSOR_OFF,
   /* directFlag      */ VIRTUAL
   );
   oldportal=DeselectPortal();
   SelectPortal(curportal);
   ClearPortal();


   DisplayTextInPortal(0,5,"Send Packet Count      : ",VNORMAL);
   DisplayTextInPortal(1,5,"Malformed Packet Count : ",VNORMAL);
   DisplayTextInPortal(2,5,"Get ECB Request Count  : ",VNORMAL);
   DisplayTextInPortal(3,5,"Get ECB Failure Count  : ",VNORMAL);
   DisplayTextInPortal(4,5,"AES Event Count        : ",VNORMAL);
   DisplayTextInPortal(5,5,"Postponed AES Event Cnt: ",VNORMAL);
   DisplayTextInPortal(6,5,"Max Configured Sockets : ",VNORMAL);
   DisplayTextInPortal(7,5,"Max Opened Sockets Cnt : ",VNORMAL);
   DisplayTextInPortal(8,5,"Open Socket Failure Cnt: ",VNORMAL);
   DisplayTextInPortal(9,5,"Listen ECB Count       : ",VNORMAL);
   DisplayTextInPortal(10,5,"ECB Cancel Failure Cnt : ",VNORMAL);
   DisplayTextInPortal(11,5,"Find Route Failure Cnt : ",VNORMAL);
	sprintf(txtbuff, "%ld", IPXdata.sendPacketCount);
   DisplayTextInPortal(0,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",IPXdata.malformedPacketCount);
   DisplayTextInPortal(1,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%ld",IPXdata.getECBRequestCount);
   DisplayTextInPortal(2,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%ld",IPXdata.getECBFailureCount);
   DisplayTextInPortal(3,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%ld",IPXdata.AESEventCount);
   DisplayTextInPortal(4,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u", IPXdata.postponedAESEventCount);
   DisplayTextInPortal(5,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",IPXdata.maxConfiguredSocketsCount);
   DisplayTextInPortal(6,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",IPXdata.maxOpenSocketsCount);
   DisplayTextInPortal(7,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",IPXdata.openSocketFailureCount);
   DisplayTextInPortal(8,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%ld",IPXdata.listenECBCount);
   DisplayTextInPortal(9,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",IPXdata.ECBCancelFailureCount);
   DisplayTextInPortal(10,35,txtbuff,VNORMAL);
   sprintf(txtbuff, "%u",IPXdata.findRouteFailureCount);
   DisplayTextInPortal(11,35,txtbuff,VNORMAL);

   UpdatePortal();
   WaitForEscape();
	PopHelpContext();
	ReselectPortal(oldportal);
	DestroyPortal(curportal);
	
	return;
	}


/**************************************************************************
** Diagnostics Menu Action Procedure
*/
int DiagMenuAct(int operation, LIST **element)
	{
	
	switch(operation)
		{
		case -1:
			EndDiagnostics(connectionID);
		   return(0);
		   break;

		case LAN_STAT:
		   LANStat();
		   break;

		case IPX_STAT:
		   IpxStat();
		   break;

		case SPX_STAT:
		    SPXStat();
		    break;

		case WS_STAT:
		    WsStat();
		    break;

		case SHELL_STAT:
			 ShellStat();
			 break;

		case LAN_CONFIG:
			 LANConfig();
			 break;

		case ADT_STAT:
			 ADTstat();
			 break;

		case GNMA_OPTIONS:
			 GNMAmenu();
			 break;
		}

   return(-1);
	}


/**************************************************************************
** Diagnostics Menu
*/
void DiagMenu(void)
	{
	WORD					count;
	unsigned int		cLimit;
	unsigned int		nRes;
	unsigned int		rT1;
	unsigned int		rT2;
	BYTE              offset;
	AllResponseData   intvptr;
	GNMAInfoStruct		GNMAdata;
	int					ccode;

	cLimit = componentList[0];
	cLimit = cLimit + 1;

	offset = (BYTE) FindComponentOffset(componentList, 9);
	ccode = GetGNMAInfo(connectionID, offset, &intvptr, &GNMAdata);
	nRes = GNMAdata.GNMANumberOfResponders;
	rT1 = GNMAdata.Type[0].ResponderType;
	rT2 = GNMAdata.Type[1].ResponderType;

   PushList();
   InitList();
   AppendToPopUpMenu(SHELL_STAT, SHELL_STAT, NO_HELP_CONTEXT);
   AppendToPopUpMenu(ADT_STAT, ADT_STAT, NO_HELP_CONTEXT);
   AppendToPopUpMenu(IPX_STAT, IPX_STAT, NO_HELP_CONTEXT);
   AppendToPopUpMenu(SPX_STAT, SPX_STAT, NO_HELP_CONTEXT);
   AppendToPopUpMenu(LAN_STAT, LAN_STAT, NO_HELP_CONTEXT);
   AppendToPopUpMenu(LAN_CONFIG, LAN_CONFIG, NO_HELP_CONTEXT);
   AppendToPopUpMenu(WS_STAT, WS_STAT, NO_HELP_CONTEXT);

	for (count=1; count < cLimit; count++)
		{
		if ((componentList[count] == 9) && (nRes > 0) && (rT1 == 1) && (rT2 == 0))
			{
          AppendToPopUpMenu(GNMA_OPTIONS, GNMA_OPTIONS,
				                NO_HELP_CONTEXT);
			}

		}

   PopUpMenu(
		/* MenuHeader       */ DIAG_MENU,
		/* centerLine y     */ 0,
		/* centerColumn x   */ 0,
		/* menuborder       */ DOUBLE,
		/* startOption      */ NullPointer(LIST),
		/* menuFlags        */ MENU_NORMAL | NO_SORT,
		/* menuHelp         */ MENU_DIAG_HELP,
		/* action procedure */ DiagMenuAct
		);

   DestroyList();
   PopList();
	}

/**************************************************************************
** Function that calls the BeginDiagnostics Function
*/
int BeginDiag(LIST *node)
	{
	int             ccode;
	int             completionCode;
	CONNID          *cid;
	WORD            diagSocket;
	
	cid=(CONNID *)node->otherInfo;
	ccode = NWGetInetAddr (cHandle, cid->connID, &internetAddr);
	if(ccode != 0) DisplayErrorCondition(GETINTERNETADDRESS, ccode, WARNING);
	diagSocket=0x456;
	memcpy(destination.network, internetAddr.networkAddr, 4);
	memcpy(destination.node, internetAddr.netNodeAddr, 6);
	memcpy(destination.socket, &diagSocket, 2);

	completionCode = BeginDiagnostics(&destination, &connectionID,	componentList);
	if(completionCode != 0)
   	{
   	DisplayErrorCondition(BEGINDIAGNOSTICS, completionCode, WARNING);
		return(1);
   	}
   
	return(0);
	}

/**************************************************************************
** LAN Information Action Procedure
*/
int LanInfoAct(int operation, LIST **element, int *line)
	{
	int	  ccode;			
	LIST    *node;
	node =  *element;

	switch(operation)
	   {
	   case M_ESCAPE:
	      return(0);
	
		case M_SELECT:
	      ccode = BeginDiag(node);
			if(ccode) return(0);
	      DiagMenu();
	      break;
	   }
	return(-1);
	}

/**************************************************************************
** Build the Connected users list
*/
int LanInfo(void)
	{
	NWNUMBER						maxConnections;
	int                     count;
	int                     ccode;
	WORD                    connectNo;
	int                     statuscode;
	int                     connections;
	char                    ObjNam[48];
	CONN_INFO               *ci;
	char                    *logbuf1 = "NOT-LOGGED-IN";
	char                    *logbuf2 = "NOT_LOGGED_IN";
	int                     ListUserInfo();
	int                     tstflg1;
	int                     tstflg2;
	
	/*-----------------------------------------------------------------------
	**	Initialize the list
	*/
	PushList();
	InitList();

	count = 0;
	connections = 0;
	statuscode = NWGetFileServerInformation (cHandle, NULL, NULL,
		                                      NULL, NULL, &maxConnections,
													     NULL, NULL, NULL, NULL, NULL);

	if(statuscode != 0) Alert(GET_SERVER_INFO_FAILED, 0, 0);

	connections = maxConnections;

	StartWait(16,40);

	/*----------------------------------------------------------------------
	** Build list of connected object
	*/
	for(count=1; count <= connections; count++)
		{
	  	connectNo = count;
	  	ccode=NWGetConnectionInformation (cHandle, connectNo, ObjNam, NULL, NULL, NULL);
	  	tstflg1 = strcmp(logbuf1, ObjNam);
	  	tstflg2 = strcmp(logbuf2, ObjNam);
	  	if(ccode == 0 && strlen(ObjNam) != 0 && tstflg1 !=0 && tstflg2 != 0)
	       {
		    ci=malloc(sizeof(CONN_INFO));
		    if(ci==NULL) break;

		    ci->conn_num=connectNo;
		    AppendToList(ObjNam, (char *)ci);
	       }
	 	}

	EndWait();

	ccode = List(
		/* Header Text   */ USER_LIST,
		/* Center Line   */ 0,
		/* Center Col    */ 15,
		/* Visible High  */ 10,
		/* Visible Width */ 25,
		/* Valid ex keys */ M_ESCAPE|M_MSELECT|M_SELECT,
		/* Element **    */ NullPointer(LIST),
		/* split         */ 0,
		/* Format Proc   */ NullProcedure(int),
		/* Action Proc   */ LanInfoAct
		);

	DestroyList();
	PopList();

	return(0);
	}

/**************************************************************************
** Change Server Action Proc. / Set preferred connection ID (1 - 20)
*/
int ChangeServerAct(int operation, LIST **element, int *line)
	{
	LIST    *node;
	WSCONNHANDLE  *wsch;

	node =  *element;

	wsch =(WSCONNHANDLE *)node->otherInfo;

	switch(operation)
	   {
	   case M_ESCAPE:
	      return(0);
	
		case M_SELECT:
			NWSetPrimaryConnectionID(wsch->wsConnHandle);
			cHandle=wsch->wsConnHandle;
			return(0);
	   }

	return(-1);
	}


/**************************************************************************
** Change server menu procedure
*/
void ChangeServer(void)
	{
	int				ccode;
	WORD				count;
	char				FSName[47+1];
	WSCONNHANDLE			*wsch;
	NWCONN_HANDLE  cHandleBuff[20];
	NWSTRUCT_SIZE  cListSize;
	NWNUMBER			cNumConnections;

	PushList();
	InitList();

	cListSize = 20;
	NWGetConnectionList(0, cHandleBuff, cListSize, &cNumConnections);

	for(count=0; count<cNumConnections; count++)
		{
			wsch = malloc(sizeof(WSCONNHANDLE));
			if(wsch == NULL) break;
			wsch->wsConnHandle = cHandleBuff[count];
			NWGetFileServerName(cHandleBuff[count], FSName);
		   AppendToList(FSName, (char *)wsch);
		}

	ccode = List(
		/* Header Text   */ SERVER_LIST,
		/* Center Line   */ 0,
		/* Center Col    */ 15,
		/* Visible High  */ 10,
		/* Visible Width */ 25,
		/* Valid ex keys */ M_ESCAPE|M_SELECT,
		/* Element **    */ NullPointer(LIST),
		/* split         */ 0,
		/* Format Proc   */ NullProcedure(int),
		/* Action Proc   */ ChangeServerAct
		);

	DestroyList();
	PopList();

	}

/**************************************************************************
** Server Information Procedure
*/
void ServerInfo(void)
	{
	int                     curportal;
	int                     oldportal;
	char                    txtbuff[80];
	char							fileServerName[47+1];

   NWSERVICE_VERSION       majorVersion, minorVersion, revision;
	NWNUMBER						maxConnections, maxConnectionsUsed;
	NWNUMBER					   connectionsInUse, numVolumes;

	NWGetFileServerInformation (cHandle, fileServerName, 
		            &majorVersion, &minorVersion, &revision, &maxConnections,
						&maxConnectionsUsed, &connectionsInUse, &numVolumes,
						NULL, NULL);

	curportal = CreatePortal(
   	/* topline         */ 5,
   	/* leftColumn      */ 5,
   	/* frameHeight     */ 14,
   	/* frameWidth      */ 65,
   	/* virtualHeight   */ 14,
   	/* virtualWidth    */ 65,
   	/* saveFlag        */ SAVE,
   	/* headerText      */ NOHEADER,
   	/* headerAttr      */ VNORMAL,
   	/* borderType      */ SINGLE,
   	/* borderAttr      */ VINTENSE,
   	/* cursorState     */ CURSOR_OFF,
   	/* directFlag      */ VIRTUAL
   	);
   oldportal=DeselectPortal();
   SelectPortal(curportal);
   ClearPortal();

   DisplayTextInPortal(0,5,"Primary Server Name: ",VNORMAL);
   DisplayTextInPortal(2,5,"NetWare Version: ",VNORMAL);
   DisplayTextInPortal(4,5,"Max number of connections: ",VNORMAL);
   DisplayTextInPortal(6,5,"Connections in use: ",VNORMAL);
   DisplayTextInPortal(8,5,"Peak Connections used: ",VNORMAL);

   sprintf(txtbuff, "%s", fileServerName);
   DisplayTextInPortal(0,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%d",maxConnections);
   DisplayTextInPortal(4,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%d",connectionsInUse);
   DisplayTextInPortal(6,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%d",maxConnectionsUsed);
   DisplayTextInPortal(8,35,txtbuff,VNORMAL);

   sprintf(txtbuff, "%d.%d revision %d", majorVersion, minorVersion, revision);
   DisplayTextInPortal(2,35,txtbuff,VNORMAL);

   UpdatePortal();
   WaitForAnyKey();
	ReselectPortal(oldportal);
	DestroyPortal(curportal);

	return;
	}

/**************************************************************************
** Main Menu Action Procedure
*/
int MenuMainAct(int option, LIST **currentOption, int *line)
	{
	switch(option)
		{
		case -1:
			if(VerifyProgramExit()) return (0);
			break;

		case SERVER_INFO:
			ServerInfo();
			break;
		
		case LAN_INFO:
			LanInfo();
			break;
	
		case CHANGE_SERVER:
			ChangeServer();
			break;
		}

	return(-1);
	}

/**************************************************************************
** Program Initialization.
*/
void Init(void)
	{
	int cCode;

	/*----------------------------------------------------------------------
	** Init C-Worthy & display program copyright.
	*/
	strcpy(_copyrightNotice1, "(C)Copyright 1994,  Novell Inc.");
	strcpy(_copyrightNotice2, "All rights reserved");
	InitUtility(PROGRAM_NAME, PROGRAM_VERSION, SHORT_PROGRAM_NAME, HEADER_NORMAL, "LANSNOOP.HLP");

	/*----------------------------------------------------------------------
	** Register an exit procedure with C-WORTHY.
	*/
	RegisterExitProcedure(Uninit);

	/*----------------------------------------------------------------------
	** Init Novell IPX library.
	*/
	cCode=IPXInitialize();
	if(cCode) DisplayErrorCondition(IPXINITIALIZE, cCode, FATAL);

	/*----------------------------------------------------------------------
	** Set Default Connection Handle IPX library.
	*/
	NWGetDefaultConnectionID(&cHandle);

	return;
	}

/**************************************************************************
** Program Start.
*/
void MenuMain(void)
	{
	InitList();
	AppendToPopUpMenu(SERVER_INFO, SERVER_INFO, NO_HELP_CONTEXT);
	AppendToPopUpMenu(LAN_INFO,LAN_INFO, NO_HELP_CONTEXT);
	AppendToPopUpMenu(CHANGE_SERVER,CHANGE_SERVER, NO_HELP_CONTEXT);

   PopUpMenu(
		/* MenuHeader       */ MENU_MAIN_HDR,
		/* centerLine y     */ 9,
		/* centerColumn x   */ 0,
		/* menuborder       */ DOUBLE,
		/* startOption      */ NullPointer(LIST *),
		/* menuFlags        */ MENU_NORMAL,
		/* menuHelp         */ MENU_MAIN_HELP,
		/* action procedure */ MenuMainAct
		);

	DestroyList();

	return;
	}

/**************************************************************************
** Program Start.
*/
void main(void)
	{
	Init();
	MenuMain();
	Uninit();
	}
